{% include 'head.html' %}
<header>
    <div class="container">
        <div class="column">
            <h1>{{ status_code }}</h1>
            <h3>{{ name }}</h3>
        </div>
    </div>
</header>
<div class="error-section">
    <div class="container">
        <div class="column">
            <blockquote>
                <h4>Oh no!</h4>
                <p>Wordlette couldn't find this page.</p>
            </blockquote>
        </div>
    </div>
</div>
<div class="container">
    <div class="column">
        <a href="/" class="go-home">Take Me Home</a>
    </div>
</div>
{% include "foot.html" %}
