<form method="post">
    <div class="row">
        <div class="column column-50">
            <fieldset>
                {% for name, field in form.fields.items() %}
                {% if name in form.labels and "label-inline" not in field.attrs["class"] %}
                {{ form.labels[name].render() }}
                {% endif %}
                {% if name in form.errors %}
                {{ field.render(add_classes=["error"]) }}
                <div class="error">*{{ form.errors[name] }}</div>
                {% else %}
                {{ field.render() }}
                {% endif %}
                {% if name in form.labels and "label-inline" in field.attrs["class"] %}
                {{ form.labels[name].render(add_classes=["label-inline"]) }}
                {% endif %}
                {% endfor %}
            </fieldset>
            <div class="float-right form-nav">
                {% for button in form.buttons %}
                {{ button.render() }}
                {% endfor %}
            </div>
        </div>
    </div>
</form>