{% include 'head.html' %}
<header class="short">
    <div class="container">
        <div class="column">
            <h1>{{ heading }}</h1>
            {% if subtitle %}
            <h3>{{ subtitle }}</h3>
            {% endif %}
        </div>
    </div>
</header>
{% if created_settings_file %}
<div class="container">
    <div class="row">
        <div class="column">
            <p><i>Wordlette</i> has saved your settings!</p>
        </div>
    </div>
    <div class="row">
        <div class="column">
            <a href="{{ next_page_url }}" class="button">Next</a>
        </div>
    </div>
</div>
<script>
    setTimeout(function () {
        window.location.href = "{{ next_page_url }}";
    }, 5000);
</script>
{% else %}
<div class="container">
    {{ form.render() }}
</div>
{% endif %}
{% include "foot.html" %}
