{% include 'head.html' %}
<header>
    <div class="container">
        <div class="column">
            <h1>{{ title }}</h1>
            {% if subtitle %}
            <h3>{{ subtitle }}</h3>
            {% endif %}
        </div>
    </div>
</header>
<div class="container">
    <div class="column">
        <h4>Welcome to <i>Wordlette</i>!</h4>
        <p>Your site needs some setup before it'll be ready to use. It'll take less than 5 minutes, so let's get
            started.</p>
        <a href="{{ next_page_url }}" class="button setup-welcome-next">Let's go</a>
    </div>
</div>
{% include "foot.html" %}
