{% include 'head.html' %}
<header>
    <div class="container">
        <div class="column">
            <h1>{{ status_code }}</h1>
            <h3>{{ name }}</h3>
        </div>
    </div>
</header>
<div class="error-section">
    <div class="container">
        <div class="column">
            <blockquote>
                {% if exception %}
                <h4>{{ exception.name }}</h4>
                <p>{{ exception.message }}</p>
                {% if exception.notes %}
                <ul>
                    {% for note in exception.notes %}
                    <li>{{ note }}</li>
                    {% endfor %}
                </ul>
                {% endif %}
                {% else %}
                <h4>Oops!</h4>
                <p>Wordlette encountered an error while loading this page.</p>
                {% endif %}
            </blockquote>
        </div>
    </div>
</div>
<div class="container">
    <div class="column">
        <a href="/" class="go-home">Take Me Home</a>
    </div>
</div>
{% if exception %}
<div class="container">
    <div class="column">
        <h4>Stacktrace</h4>
        <pre>{{ exception.stacktrace }}</pre>
    </div>
</div>
{% endif %}
{% include "foot.html" %}
